@@ -74,6 +74,10 @@ sub new {
     return $obj;
 } #new
 
+sub new_with_same_permissions {
+    return new Yote::Obj();
+} #new_with_same_permissions
+
 #
 # Called the very first time this object is created. It is not called
 # when object is loaded from storage.
@@ -190,6 +194,11 @@ sub _check_access_update {
     return 1;
 } #_check_access
 
+sub _container_type {
+    my( $self, $args ) = @_;
+    return Yote::ObjProvider::container_type( $self->{ID}, $args );
+} #_container_type
+
 sub _count {
     my( $self, $args ) = @_;
     if( ref( $args ) ) {
@@ -346,6 +355,12 @@ sub add_to {
     return $self->_add_to( $listname, @$items );
 } #add_to
 
+sub container_type {
+    my( $self, $data, $account ) = @_;
+    die "Access Error" unless $self->_check_access( $account, 0, ref( $data ) ? $data->{ name } : $data );
+    return $self->_container_type( $data );
+} #container_type
+
 sub count {
     my( $self, $data, $account ) = @_;
     die "Access Error" unless $self->_check_access( $account, 0, ref( $data ) ? $data->{ name } : $data );
@@ -682,6 +697,10 @@ This method is called each time an object is loaded from the data store.
 
 Adds the items to the list attached to this object specified by name.
 
+=item container_type( container_name )
+
+returns the class name of the given container from this host object.
+
 =item count( field_name )
 
 Returns the number of items for the field of this object provided it is an array or hash.
@@ -710,6 +729,10 @@ Removes the item at the index postion from the list attached to this object spec
 
 Returns item at the index postion from the list attached to this object specified by name.
 
+=item new_with_same_permissions()
+
+Returns a new yote object with the same permissions as this.
+
 =item paginate( args )
 
 Returns a paginated list or hash. Arguments are
