@@ -64,6 +64,12 @@ sub commit_transaction {
     return $DATASTORE->commit_transaction();
 }
 
+sub container_type {
+    my( $host_id, $container_name ) = @_;
+    return '' unless $host_id;
+    return $DATASTORE->container_type( $host_id, $container_name ) || '';
+} #container_type
+
 sub count {
     my( $container_id, $args ) = @_;
     return 0 unless $container_id;
@@ -559,6 +565,10 @@ which also serves as the locker. The datalocker is responsible for locking and u
 
 Requests the data store used commit the transaction.
 
+=item container_type( host_id, container_name )
+
+returns the class name of the given container from a host class.
+
 =item count( container_id, args )
 
 Returns the number of items in the given container. Args are optional and are
