@@ -19,16 +19,28 @@ sub _init {
     $self->get_list( [ 1, "Bagel" ] );
     $self->get_hash( { one=>1, food => "Bagel" } );
     $self->get_my_hash( { store => { AnObject => new Yote::Obj( { flavor => 'blueberry' } ) } } );
-    $self->set_foo_array( [ "A".."Z" ] );
+
+    my( @arry ) = ( "A" .. "Z" );
+    $self->set_foo_array( [ @arry ] );
+    $self->set_foo_hash( { map { $_ => $arry[$_] } ( 0..$#arry ) } );
+    $self->set__foo_array( [ @arry ] );
+    $self->set__foo_hash( { map { $_ => $arry[$_] } ( 0..$#arry ) } );
 }
 
 sub _load {
     my $self = shift;
     $self->set_my_hash( { store => { AnObject => new Yote::Obj( { flavor => 'blueberry' } ) } } );
-    $self->get_foo_array( [ "A".."Z" ] );
-    $self->get_bar_hash( { map { $_ => 1 } ( "A".."Z" ) } );
+    my( @arry ) = ( "A" .. "Z" );
+    $self->get_foo_array( [ @arry ] );
+    $self->get_foo_hash( { map { $_ => $arry[$_] } ( 0..$#arry ) } );
+    $self->get__foo_array( [ @arry ] );
+    $self->get__foo_hash( { map { $_ => $arry[$_] } ( 0..$#arry ) } );
+    $self->get_bar_hash( { map { $_ => 1 }  @arry } );
 }
 
+# needed to test non-room access of underscore variables.
+sub _check_access { return 1; }
+
 sub hello {
     my( $self, $data, $acct ) = @_;
     $self->set_count( 1 + $self->get_count( 0 ) );
